
/* area.t */

module data;

/* ---------------------------------------------------------------------- */

idecl
{
   #include <QGraphicsScene>
   #include <QGraphicsItemGroup>
   #include <QGraphicsRectItem>
   #include <QGraphicsLineItem>

   // #include <strings.h>
   // using namespace std;
}

decl
{
}

/* ---------------------------------------------------------------------- */

class Area : public QGraphicsRectItem
{
   attr <QString> Title;

   list <Area> Items
   {
      Suffix1 = Item;
      Suffix2 = Items;
      Up = aboveArea;
      insert_notification = connected; // notify item
      remove_notification = disconnected;
   }

   list <Source> Sources
   {
      Suffix1 = Source;
      Suffix2 = Sources;
      Up = area;
      insert_notification = connected;
      remove_notification = disconnected;
   }

   list <Target> Targets
   {
      Suffix1 = Target;
      Suffix2 = Targets;
      Up = area;
      insert_notification = connected; // notify item
      remove_notification = disconnected;
   }

   list <Field> Fields
   {
      Suffix1 = Field;
      Suffix2 = Fields;
      Up = aboveArea;
   }

   decl
   {
      private:
         void initialize ();

         void connected ();
         void disconnected ();

      public:
         void addArea (Area * a) { insertLastItem (a); }
         void addSource (Source * s) { insertLastSource (s); }
         void addTarget (Target * t) { insertLastTarget (t); }
         void addField (Field * f) { insertLastField (f); }
         void initializeTopLevel ();

     public:
         void updateConnections ();
         void updateTargets (Area * target_area);
         virtual QVariant itemChange (GraphicsItemChange change, const QVariant & value);

     protected:
         void dragEnterEvent (QGraphicsSceneDragDropEvent * event);
         void dropEvent (QGraphicsSceneDragDropEvent * event);
   }

   init
   {
       initialize ();
   }
};

/* ---------------------------------------------------------------------- */

class Source : public QGraphicsRectItem
{
   list <Connection> Connections
   {
      Suffix1 = Connection;
      Suffix2 = Connections;
      Up = source;
      insert_notification = connected; // notify item
      remove_notification = disconnected;
   }

   decl
   {
      private:
         void initialize ();
         void connected ();
         void disconnected ();
   }

   init
   {
       initialize ();
   }
};

class Target : public QGraphicsRectItem
{
   decl
   {
      private:
         void initialize ();
         void connected ();
         void disconnected ();
   }

   init
   {
       initialize ();
   }
};

class Connection
{
   attr < Target * > target;

   decl
   {
      public:
         void setSource (Source * src) { src->insertLastConnection (this); }

      private:
         void initialize ();
         void connected ();
         void disconnected ();

      private:
         QGraphicsItemGroup * group;
         qreal x1;
         qreal y1;
         QPen pen;
         void addLine (qreal x2, qreal y2);
         void horizLine (qreal x2);
         void vertLine (qreal y2);

      public:
         void updateConnection ();
   }

   init
   {
       initialize ();
   }
};

/* ---------------------------------------------------------------------- */

class Field
{
   attr <QString> Name;
   attr <QString> Value;

   public:
      virtual QString getText () { return ""; }
      virtual void setText (QString txt) { }

      virtual bool ReadOnly () { return false; }
      virtual bool DialogEnabled () { return false; }
      virtual void DialogExecute () { }
      virtual bool ListEnabled () { return false; }
      virtual int ListCount () { return 0; }
      virtual QString ListValue (int inx) { return ""; }
      virtual void ListClick (int inx) { }
};

/* ---------------------------------------------------------------------- */
